% measureNaPactivation.m
% Jessica Parker, October 16, 2024
%
% This Matlab script loads the simulation data made by the C code and measures the steady-state activation level 
% at an array of different membrane potentials, which the cell is clamped at for run6 = 2 and which are indexed by run5.

clear all;
close all;

vrsn = 'A';

run1 = 3;
run2 = 0;
run3 = 0;
run4 = 2;
run5i = 1;
run5f = 55;
run6i = 1;
run6f = 2;

dir1 = [num2str(run1) '_' num2str(run2) '_' num2str(run3)];
dir2 = [dir1 '_' num2str(run4)];
leakDir = [dir1 '_1']; % Data in 3_0_0_1 is used to measure leak current using P/4 leak subtraction

dataDir = 'data/';
prfx = 'NaiMnap';
nvar = 2;
Naindx = 1;
mNaPindx = 2;
VclampStrt = -70; % The voltage set for run5 = 1 and run6 = 2.
VclampStp = 1; % The voltage increase between run5 instances
VclampIntl = -90; % The holding potential used in 3_0_0_0
VclampInter = -80; % The starting potental when run6 = 1
VclampInterL = 0.25*(VclampInter - VclampIntl) + VclampIntl; % Interim membrane potential for leak current simulation in 3_0_0_1 when run6 = 1 (-87.5 mV)
tint = 0.0001;

fntnm = 'Lucida Sans';
fntsz = 14;
vsblty = 'on';
CloseFigs = 1;
PlotTraces = 1;
strtPltTrc = 0; % First run5 instance plotted if PlotTraces is set to 1
stpPltTrc = 50; % Last run5 instance plotted if PlotTraces is set to 1
skpPltTrc = 10; % Number of run5 instances (+1) to skip between traces plotted. To skip none, set to 1.
SetMeasureTime = 0; % Set to 0 in order to measure the actual peak current after depolarization step, set to 1 to estimate this peak current by
                    % measuring at a specified time (msrTm) after the step
msrTm = 0.034; % Used if SetMeasureTime is set to 1.
msrTm0 = 0.034; % Only used if max gNaP does not occur shortly after voltage step and SetMeasureTime is set to 0

readpars; % Reading parameter values in readpars.m
assmdENa = 45.0; % This is assumed Na reversal potential used to measure gNaP

VarN = ['../' dir1 '_0/' dataDir prfx dir1 '_0_0_0.dat'];
fl = fopen(VarN);
yyStrt = fread(fl,[nvar, 10000000],'double')'; % Contains simulation data from 3_0_0_0 where it is held at the holding potential of -90 mV
fclose(fl);

for run5 = run5i:run5f
    yy = yyStrt; % starts at -90 mV with data from 3_0_0_0
    leakyy = yyStrt;
    indxs(1) = length(yy);
    leakIndxs(1) = length(leakyy);
    for run6=run6i:run6f   % Concatenating portions of time saved in separate files

        dir3 = [dir2 '_' num2str(run5)];

        VarN = [dataDir prfx dir3 '_' num2str(run6) '.dat'];
        fl = fopen(VarN);
        yy0 = fread(fl,[nvar, 10000000],'double')'; % Reading simulation data from 3_0_0_2
        fclose(fl);

        yy = [yy(1:end-1,:); yy0]; % Remove last point here, because otherwise this time point will be repeating as the first time point of the next file
        indxs(run6+1) = length(yy);

        leakFlN = ['../' leakDir '/' dataDir prfx leakDir '_' num2str(run5) '_' num2str(run6) '.dat'];
        leakFl = fopen(leakFlN);
        leakyy0 = fread(leakFl,[nvar, 10000000],'double')'; % Reading simulation data from 3_0_0_1
        fclose(leakFl);

        leakyy = [leakyy(1:end-1,:); leakyy0];
        leakIndxs(run6+1) = length(leakyy);
    end

    Vclamp = VclampStrt + VclampStp*(run5-1); % Final membrane potential step for this run5 instance
    VclampL = 0.25*(Vclamp - VclampIntl) + VclampIntl; % Final membrane potential step for this run5 instance in 3_0_0_1 for leak current simulation
    Vclamps(run5-run5i+1) = Vclamp; % Array of final Vclamp steps for all run5 instances
    Vm = [VclampIntl*ones(indxs(1),1); VclampInter*ones(indxs(2)-indxs(1),1); ... 
        Vclamp*ones(indxs(3)-indxs(2),1)]; % All membrane potential steps for this run5 instance
    leakVm = [VclampIntl*ones(indxs(1),1); VclampInterL*ones(indxs(2)-indxs(1),1); ... 
        VclampL*ones(indxs(3)-indxs(2),1)]; % The membrane potential protocol used to measure the leak current in 3_0_0_1 for P/4 leak subtraction

    Ipump = Ipmpmx./(1.0+(Naih*exp(alpha*Vm/rtf)./yy(:,Naindx)).^nhillNa); % Ipump in 3_0_0_2
    Inap = gNaP*yy(:,mNaPindx).*(Vm-rtf*log(Nae./yy(:,Naindx))); % INaP in 3_0_0_2
    IleakNa = gL*(EL-EK)/(ENa-EK)*(Vm-rtf*log(Nae./yy(:,Naindx))); % ILeakNa in 3_0_0_2
    IleakK = gL*(EL-ENa)/(EK-ENa)*(Vm-EK); % ILeakK in 3_0_0_2
    Ileak = IleakNa + IleakK;
    Itot = Ipump + Inap + Ileak;

    IleakPump = Ipmpmx./(1.0+(Naih*exp(alpha*leakVm/rtf)./leakyy(:,Naindx)).^nhillNa); % Ipump in 3_0_0_1
    IleakNap = gNaP*leakyy(:,mNaPindx).*(leakVm-rtf*log(Nae./leakyy(:,Naindx))); % INaP in 3_0_0_1
    IleakLNa = gL*(EL-EK)/(ENa-EK)*(leakVm-rtf*log(Nae./leakyy(:,Naindx))); % ILeakNa in 3_0_0_1
    IleakLK = gL*(EL-ENa)/(EK-ENa)*(leakVm-EK); % ILeakK in 3_0_0_1
    IleakL = IleakLNa + IleakLK;
    IleakMsrd0 = IleakPump + IleakNap + IleakL;
    IleakMsrd = IleakMsrd0;
    IleakIntl = mean(IleakMsrd0(round(indxs(1)/2):indxs(1))); % The steady-state leak current after the potential has been held at -90 mV for a while
    IleakMsrd(indxs(1)+1:indxs(end)) = IleakIntl + 4.0*(IleakMsrd0(indxs(1)+1:indxs(end)) - IleakIntl); % Estimated leak current used for P/4 leak subtraction

    tt = 0.0:tint:tint*(indxs(end)-1);

    iMsr = length(indxs)-1;
    InapMsrd = Itot - IleakMsrd; % Subtracting measured leak current
    gnapMsrd = InapMsrd./(Vm - assmdENa); % Measuring gNaP using assumed Na reversal potential
    if SetMeasureTime
        maxGnapIndx = round(msrTm/tint) + indxs(iMsr);
        maxGnap = gnapMsrd(maxGnapIndx); % Estimating peak gNaP using time of expected peak
    else
        [maxGnap, maxGnapIndx0] = max(gnapMsrd(indxs(iMsr)+1:indxs(iMsr)+round(1.8*msrTm0/tint))); % Finding peak gNaP
        maxGnapIndx = maxGnapIndx0 + indxs(iMsr); % index of peak gNaP
        if maxGnapIndx-indxs(iMsr) > 1.7*round(msrTm0/tint) || maxGnapIndx-indxs(iMsr) < round(0.1*msrTm0/tint) % If the maximum current does not occur
            % shortly after the depolarization step
            maxGnapIndx = round(msrTm0/tint)+indxs(iMsr); % Use expected time of peak (msrTm0)
            maxGnap = gnapMsrd(maxGnapIndx); % gNaP at expected time of peak
            disp(['Setting measurement time to ' num2str(msrTm0) ' ms for run5 = ' num2str(run5)]);
        end
    end
    allGnaps(run5-run5i+1) = maxGnap; % array of max gNaPs for all run5 instances
    InapHere(run5-run5i+1) = InapMsrd(maxGnapIndx); % used to plot Inap at time of peak versus membrane potential
    InapEnd(run5-run5i+1) = InapMsrd(end); % used to plot steady-state Inap versus membrane potential
    msrTms(run5-run5i+1) = maxGnapIndx-indxs(iMsr); % time of peak gNaP

    if (PlotTraces && run5 >= strtPltTrc) && (run5 <= stpPltTrc && mod(run5,skpPltTrc) == 0)
        xmin = tt(indxs(iMsr)+1)-0.1;
        xmax = tt(indxs(iMsr)+1) + 0.2;

        f = figure('visible',vsblty);
        f.PaperPositionMode = 'manual';
        f.PaperUnits = 'inches';
        f.Units = 'inches';
        f.OuterPosition = [1 1 8.0 4.0];
        f.InnerPosition = [0.25 0.25 7.5 3.5];
        f.PaperPosition = [0.25 0.25 7.0 3.0];
        f.RendererMode = 'manual';

        axes('position',[0.15 0.6 0.8 0.35]);
        hold on;
        plot(tt, InapMsrd, 'k', 'linewidth', 1.5);
        ylabel('I_{NaP} (pA)');
        xticks([]);
        xlim([xmin xmax]);
        box off;
        ax = gca;
        ax.FontName = fntnm;
        ax.FontSize = fntsz;
        ax.LineWidth = 2.0;

        axes('position',[0.15 0.2 0.8 0.35]);
        hold on;
        plot(tt, gnapMsrd, 'k', 'linewidth', 1.5);
        plot(tt(maxGnapIndx),maxGnap,'rx','linewidth',1.5,'markersize',10);
        ylabel('g_{NaP} (nS)');
        xlabel('Time (s)');
        xlim([xmin xmax]);
        box off;
        ax = gca;
        ax.FontName = fntnm;
        ax.FontSize = fntsz;
        ax.LineWidth = 2.0;

        print(f,['plots/Vclamp' dir2 '_' num2str(run5) vrsn '.eps'],'-depsc','-r0');

        if strcmp(vsblty,'off')
            close(f);
        elseif CloseFigs
            close(f);
        end
    end
end

mxmlGnap = max(allGnaps); % Maximum gNaP measured for all run5 instances
mnapMsrd = allGnaps/mxmlGnap; % Measuring mNaP by dividing gNaP at the current peak for each run5 instance by the
% maximum gNaP measured among all run5 instances

f = figure('visible',vsblty);
f.PaperPositionMode = 'manual';
f.PaperUnits = 'inches';
f.Units = 'inches';
f.OuterPosition = [1 1 5.0 5.0];
f.InnerPosition = [0.25 0.25 4.5 4.5];
f.PaperPosition = [0.25 0.25 4.0 4.0];
f.RendererMode = 'manual';

axes('position',[0.2 0.2 0.75 0.65]);
hold on;
plot(Vclamps,mnapMsrd,'k','linewidth',1.5);
xlabel('V_{m} (mV)');
ylabel('m_{NaP}');
title(['Assumed E_{Na} = ' num2str(assmdENa) ' mV']);
box off;
ax = gca;
ax.FontName = fntnm;
ax.FontSize = fntsz;
ax.LineWidth = 2.0;

print(f,['plots/mnapMsrd' dir2 vrsn '.eps'],'-depsc','-r0');

if strcmp(vsblty,'off')
    close(f);
elseif CloseFigs
    close(f);
end

f = figure('visible',vsblty);
f.PaperPositionMode = 'manual';
f.PaperUnits = 'inches';
f.Units = 'inches';
f.OuterPosition = [1 1 5.5 5.0];
f.InnerPosition = [0.25 0.25 5.0 4.5];
f.PaperPosition = [0.25 0.25 4.5 4.0];
f.RendererMode = 'manual';

axes('position',[0.2 0.2 0.75 0.75]);
hold on;
plot(Vclamps,InapHere,'k','linewidth',1.5);
xlabel('V_{m} (mV)');
ylabel('I_{NaP} at Measurement (pA)');
box off;
ax = gca;
ax.FontName = fntnm;
ax.FontSize = fntsz;
ax.LineWidth = 2.0;

print(f,['plots/InapAtMeasurement' dir2 vrsn '.eps'],'-depsc','-r0');

if strcmp(vsblty,'off')
    close(f);
elseif CloseFigs
    close(f);
end

f = figure('visible',vsblty);
f.PaperPositionMode = 'manual';
f.PaperUnits = 'inches';
f.Units = 'inches';
f.OuterPosition = [1 1 5.5 5.0];
f.InnerPosition = [0.25 0.25 5.0 4.5];
f.PaperPosition = [0.25 0.25 4.5 4.0];
f.RendererMode = 'manual';

axes('position',[0.2 0.2 0.75 0.75]);
hold on;
plot(Vclamps,InapEnd,'k','linewidth',1.5);
xlabel('V_{m} (mV)');
ylabel('Steady-State I_{NaP} (pA)');
box off;
ax = gca;
ax.FontName = fntnm;
ax.FontSize = fntsz;
ax.LineWidth = 2.0;

print(f,['plots/InapSteadyState' dir2 vrsn '.eps'],'-depsc','-r0');

if strcmp(vsblty,'off')
    close(f);
elseif CloseFigs
    close(f);
end

dlmwrite([dataDir 'mnapMsrd' dir2 vrsn '.txt'],[mnapMsrd; Vclamps; allGnaps]);
dlmwrite([dataDir 'maximalGnap' dir2 vrsn '.txt'],mxmlGnap);

